/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* VOID:STRING,BOXED (../vte-0.68.0/src/marshal.list:1) */
G_GNUC_INTERNAL
void _vte_marshal_VOID__STRING_BOXED (GClosure     *closure,
                                      GValue       *return_value,
                                      guint         n_param_values,
                                      const GValue *param_values,
                                      gpointer      invocation_hint,
                                      gpointer      marshal_data);
G_GNUC_INTERNAL
void _vte_marshal_VOID__STRING_BOXEDv (GClosure *closure,
                                       GValue   *return_value,
                                       gpointer  instance,
                                       va_list   args,
                                       gpointer  marshal_data,
                                       int       n_params,
                                       GType    *param_types);

/* VOID:STRING,UINT (../vte-0.68.0/src/marshal.list:2) */
G_GNUC_INTERNAL
void _vte_marshal_VOID__STRING_UINT (GClosure     *closure,
                                     GValue       *return_value,
                                     guint         n_param_values,
                                     const GValue *param_values,
                                     gpointer      invocation_hint,
                                     gpointer      marshal_data);
G_GNUC_INTERNAL
void _vte_marshal_VOID__STRING_UINTv (GClosure *closure,
                                      GValue   *return_value,
                                      gpointer  instance,
                                      va_list   args,
                                      gpointer  marshal_data,
                                      int       n_params,
                                      GType    *param_types);

/* VOID:UINT,UINT (../vte-0.68.0/src/marshal.list:3) */
G_GNUC_INTERNAL
void _vte_marshal_VOID__UINT_UINT (GClosure     *closure,
                                   GValue       *return_value,
                                   guint         n_param_values,
                                   const GValue *param_values,
                                   gpointer      invocation_hint,
                                   gpointer      marshal_data);
G_GNUC_INTERNAL
void _vte_marshal_VOID__UINT_UINTv (GClosure *closure,
                                    GValue   *return_value,
                                    gpointer  instance,
                                    va_list   args,
                                    gpointer  marshal_data,
                                    int       n_params,
                                    GType    *param_types);


G_END_DECLS
